@@ -5,8 +5,8 @@ from .main import DynamoCrudRepository  # noqa: F401
 from .main import DynamoDBGlobalParameters  # noqa: F401
 from .main import Repository  # noqa: F401
 from .main import dynamo_entity  # noqa: F401; noqa: F401
-from .simulated_dynamodb import SimulatedDynamoDBResource  # noqa: F401
-from .simulated_dynamodb import SimulatedTable  # noqa: F401
+from .main import SimulatedDynamoDBResource  # noqa: F401
+from .main import SimulatedTable  # noqa: F401
 
-__version__ = "0.1.8"
+__version__ = "0.1.10"
 __author__ = """DRUID DEVAX"""
