@@ -8,8 +8,200 @@ from dataclasses import dataclass, field, fields, is_dataclass
 from decimal import Decimal
 from enum import Enum
 from typing import Any, Generic, Iterable, TypeVar, Union
-
 import boto3
+import uuid
+
+
+class TableInterface(metaclass=abc.ABCMeta):
+    @abc.abstractmethod
+    def delete(self):
+        pass
+
+    @abc.abstractmethod
+    def scan(self):
+        pass
+
+    @abc.abstractmethod
+    def delete_item(self, Key: dict):
+        pass
+
+    @abc.abstractmethod
+    def get_item(self, Key: dict):
+        pass
+
+    @abc.abstractmethod
+    def query(self, **kwargs):
+        pass
+
+    @abc.abstractmethod
+    def put_item(self, Item: dict):
+        pass
+
+
+class BatchWriterInterface(metaclass=abc.ABCMeta):
+    @abc.abstractmethod
+    def delete_item(self, Key: dict):
+        pass
+
+
+@dataclass
+class SimulatedTable(TableInterface):
+    items: dict = field(default_factory=lambda: {})
+    table_status: str = "ACTIVE"
+    # for items with no specified sk
+    std_sk: str = field(default_factory=lambda: str(uuid.uuid1()))
+    pk_name: str = "pk"
+    sk_name: str = "sk"
+    writer: BatchWriterInterface = None
+
+    @dataclass
+    class SimulateBatchWriter(BatchWriterInterface):
+        table: TableInterface
+
+        def __init__(self, table: TableInterface = None):
+            self.table = table
+
+        def delete_item(self, Key: dict):
+            self.table.delete_item(Key)
+
+        def put_item(self, Item: dict):
+            self.table.put_item(Item)
+
+        def close(self):
+            pass
+
+        def __enter__(self):
+            return self
+
+        def __exit__(self, exc_type, exc_val, exc_tb):
+            self.close()
+
+    def _inject_sk(self, Key: dict):
+        if self.sk_name not in Key:
+            Key[self.sk_name] = self.std_sk
+
+    def _keys_from_item(self, item):
+        Key = {}
+        if self.pk_name not in item:
+            raise KeyError(
+                "The provide item must have a value for the partition key: ",
+                self.pk_name,
+            )
+        Key[self.pk_name] = item[self.pk_name]
+        if self.sk_name in item:
+            Key[self.sk_name] = item[self.sk_name]
+        else:
+            self._inject_sk(Key)
+
+        return Key
+
+    def _expression_att_val2Key(self, ExpressionAttributeValues: dict):
+        Key = {}
+        for dirty_key in ExpressionAttributeValues:
+            clean_key = dirty_key[1:-3]
+            Key[clean_key] = ExpressionAttributeValues[dirty_key]
+        return Key
+
+    def batch_writer(self):
+        if self.writer is None:
+            self.writer = self.SimulateBatchWriter(table=self)
+        return self.writer
+
+    def delete(self):
+        self.items = {}
+        self.table_status = "DELETING"
+
+    def _items_as_list(self):
+        item_list = []
+        for (
+            _,
+            partition,
+        ) in self.items.items():  # The first ignored key is the partition key
+            for _, item in partition.items():  # The second ignored key is the sort key
+                item_list.append(item)
+        return item_list
+
+    def scan(self):
+        scan_response = {
+            "Count": len(self._items_as_list()),
+            "Items": self._items_as_list(),
+        }
+        return scan_response
+
+    def delete_item(self, Key: dict):
+        aux = Key
+        Key = aux.copy()
+        self._inject_sk(Key)
+        self.items[str(Key[self.pk_name])].pop(str(Key[self.sk_name]))
+
+    def get_item(self, Key: dict):
+        aux = Key
+        Key = aux.copy()
+        self._inject_sk(Key)
+        try:
+            return {"Item": self.items[str(Key[self.pk_name])][str(Key[self.sk_name])]}
+        except Exception as ex:
+            print(ex)
+            return None
+
+    def query(self, **kwargs):
+        Key = self._expression_att_val2Key(kwargs["ExpressionAttributeValues"])
+        response = {"Items": []}
+        if self.sk_name in Key:  # One item
+            item = self.get_item(Key)
+            if item is not None:
+                response["Items"].append(item)
+        else:  # Multiple items
+            for sk in self.items[str(Key[self.pk_name])]:
+                response["Items"].append(self.items[str(Key[self.pk_name])][sk])
+        return response
+
+    def put_item(self, Item: dict):
+        Key = self._keys_from_item(Item)
+        if str(Key[self.pk_name]) not in self.items:
+            self.items[str(Key[self.pk_name])] = {}
+        self.items[str(Key[self.pk_name])][str(Key[self.sk_name])] = Item
+
+
+class SimulatedDynamoDBResource:
+    tables: dict = {}
+
+    def create_table(self, **kwargs):
+        table_name = kwargs["TableName"]
+        key_schema = kwargs["KeySchema"]
+        pk = None
+        sk = None
+        if table_name in self.tables:
+            if self.tables[table_name].table_status != "DELETING":
+                raise RuntimeError(
+                    "A table with the name ", table_name, " already exists"
+                )
+            else:
+                self.tables.pop(table_name)
+        for key_schema_dict in key_schema:
+            if key_schema_dict["KeyType"] == "HASH":
+                pk = key_schema_dict["AttributeName"]
+
+            elif key_schema_dict["KeyType"] == "RANGE":
+                sk = key_schema_dict["AttributeName"]
+
+        if pk is not None:
+            if sk is not None:
+                self.tables[table_name] = SimulatedTable(pk_name=pk, sk_name=sk)
+            else:
+                self.tables[table_name] = SimulatedTable(pk_name=pk)
+            return self.tables[table_name]
+
+        else:
+            raise RuntimeError("Provided KeySchema missing the partition key")
+
+    def Table(self, table_name: str):
+        if (table_name is None) or (table_name not in self.tables):
+            raise RuntimeError(
+                "Provided table name does not correspond to a valid table"
+            )
+        return self.tables[table_name]
+
 
 ###### ---------- Global parameters ---------- ######
 
@@ -144,6 +336,7 @@ class DynamoCrudRepository(CrudRepository):
     map_dict: map = None
     map_filled: bool = False
     print_debug: bool = False
+    create_table_if_not_exists: bool = False
 
     @staticmethod
     def ITEM2INSTANCE():
@@ -163,13 +356,13 @@ class DynamoCrudRepository(CrudRepository):
         drop_table: bool = False,
         print_debug: bool = False,
         use_mock_db: bool = False,
+        create_table_if_not_exists: bool = True
     ):
         self.entity_type = entity_type
         self.print_debug = print_debug
+        self.create_table_if_not_exists = create_table_if_not_exists
         global_values = DynamoDBGlobalParameters.get_instance()
         if use_mock_db:
-            from simulated_dynamodb import SimulatedDynamoDBResource
-
             self.ddb = SimulatedDynamoDBResource()
         else:
             try:
@@ -184,7 +377,8 @@ class DynamoCrudRepository(CrudRepository):
                     )
                 else:
                     self.ddb = boto3.resource("dynamodb")
-            except Exception:
+            except Exception as ex:
+                print(ex)
                 raise ConnectionRefusedError("Not able to connect to DynamoDB")
 
         self.map_dict = self.entity_type.dynamo_map()
@@ -198,8 +392,10 @@ class DynamoCrudRepository(CrudRepository):
             self.table_name = self.entity_type.dynamo_table_name()
         try:
             self.table = self.ddb.Table(self.table_name)
-        except Exception:
-            self.create_table()
+        except Exception as ex:
+            print(ex)
+            if self.create_table_if_not_exists:
+                self.create_table()
 
         if drop_table:
             try:
@@ -208,19 +404,20 @@ class DynamoCrudRepository(CrudRepository):
                 ):
                     self.if_debug_print("Table found and will be deleted")
                     self.table.delete()
-            except Exception:
-                pass
+            except Exception as ex:
+                print(ex)
 
         try:
             if (
                 not hasattr(self.table, "table_status")
                 or self.table.table_status == "DELETING"
-            ):
+            ) and self.create_table_if_not_exists:
                 self.if_debug_print("Table not found and will be created")
                 self.create_table()
         # If the table is not created, the interpreter issues an exception when we try to access the item_count
         except Exception:
-            self.create_table()
+            if self.create_table_if_not_exists:
+                self.create_table()
 
     def create_table(self):
         global_values = DynamoDBGlobalParameters.get_instance()
@@ -237,14 +434,16 @@ class DynamoCrudRepository(CrudRepository):
                     "WriteCapacityUnits": global_values.write_capacity_units,
                 },
             )
-        except Exception:
+        except Exception as ex:
+            print(ex)
             raise ResourceWarning("Not able to create table", self.table_name)
         return self.table
 
     def count(self):
         try:
             scan_response = self.table.scan()
-        except Exception:
+        except Exception as ex:
+            print(ex)
             raise ResourceWarning("Not able to scan table", self.table_name)
         i = scan_response["Count"]
         while "LastEvaluatedKey" in scan_response:
@@ -252,7 +451,8 @@ class DynamoCrudRepository(CrudRepository):
                 scan_response = self.table.scan(  # TableName='SingleTable',
                     ExclusiveStartKey=scan_response["LastEvaluatedKey"]
                 )
-            except Exception:
+            except Exception as ex:
+                print(ex)
                 warnings.warn("Not able to scan table" + str(self.table_name))
                 return i
             i += scan_response["Count"]
@@ -261,7 +461,8 @@ class DynamoCrudRepository(CrudRepository):
     def remove_by_keys(self, keys: dict):
         try:
             self.table.delete_item(Key=keys)
-        except Exception:
+        except Exception as ex:
+            print(ex)
             warnings.warn(
                 "Not able to delete item with keys"
                 + str(keys)
@@ -414,7 +615,8 @@ class DynamoCrudRepository(CrudRepository):
 
         try:
             scan_response = self.table.scan()
-        except Exception:
+        except Exception as ex:
+            print(ex)
             warnings.warn("Not able to scan table" + str(self.table_name))
             return None
 
@@ -430,7 +632,8 @@ class DynamoCrudRepository(CrudRepository):
                 scan_response = self.table.scan(  # TableName='SingleTable',
                     ExclusiveStartKey=scan_response["LastEvaluatedKey"]
                 )
-            except Exception:
+            except Exception as ex:
+                print(ex)
                 warnings.warn("Not able to scan table" + str(self.table_name))
                 return entity_list
             if "Items" in scan_response:
@@ -504,7 +707,8 @@ class DynamoCrudRepository(CrudRepository):
         item_params = self.instance2item_params(obj)
         try:
             self.table.put_item(Item=item_params)
-        except Exception:
+        except Exception as ex:
+            print(ex)
             warnings.warn(
                 "Not able to put item"
                 + str(item_params)
@@ -525,7 +729,8 @@ class DynamoCrudRepository(CrudRepository):
                             sk: entity.__getattribute__(sk),
                         }
                     )
-        except Exception:
+        except Exception as ex:
+            print(ex)
             raise ResourceWarning(
                 "Not able to remove all items from table", self.table_name
             )
@@ -536,7 +741,8 @@ class DynamoCrudRepository(CrudRepository):
             with self.table.batch_writer() as batch:
                 for keys in keys_list:
                     batch.delete_item(Key=keys)
-        except Exception:
+        except Exception as ex:
+            print(ex)
             raise ResourceWarning(
                 "Not able to remove items with keys",
                 keys_list,
@@ -555,7 +761,8 @@ class DynamoCrudRepository(CrudRepository):
             response = self.table.get_item(Key=keys)
             if "Item" in response:
                 return True
-        except Exception:  # Check if the keys do not compose a unique key
+        except Exception as ex:  # Check if the keys do not compose a unique key
+            print(ex)
             key_cond_exp, exp_att_val = self.keys2KeyConditionExpression(keys)
             try:
                 response = self.table.query(
@@ -813,7 +1020,7 @@ if __name__ == "__main__":
     counter = Counter()
     # To use local dynamo call DynamoDBGlobalParameters.get_instance(use_aws_cli_credentials=False)
     sample_crud = DynamoCrudRepository(
-        SampleEntityClass, drop_table=True, print_debug=True, use_mock_db=True
+        SampleEntityClass, print_debug=True, use_mock_db=True, drop_table=True, create_table_if_not_exists=True
     )
 
     aMap = SampleEntityClass.dynamo_map()
